// -*- C++ -*-
/*!
 * @file TimeMeasure.h
 * @brief Periodic time measurement class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_TIMEMEASURE_H
#define COIL_TIMEMEASURE_H

#include <vector>
#include <coil/TimeValue.h>

namespace coil
{
  
  /*!
   * @if jp
   *
   * @class TimeMeasure
   * @brief TimeMeasure NX
   *
   * ̃NX́AR[hsԂ̓vׂɎgp܂B
   * get_stat gpăR[hs̍őEŏEρEW΍Ԃvł܂B
   *
   * @else
   *
   * @class TimeMeasure
   * @brief TimeMeasure class
   *
   * This class is used for getting statistics of code execution time. 
   * Using get_stat you can get maximum, minimum, mean and standard
   * deviation time for code execution.
   *
   * @endif
   */
  class TimeMeasure
  {
  public:
    /*!
     * @if jp
     *
     * @brief ԓvp\
     *
     * @else
     *
     * @brief Structure for time statistics
     *
     * @endif
     */
    struct Statistics
    {
      double max_interval;
      double min_interval;
      double mean_interval;
      double std_deviation;
    };

    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * ԓṽvt@CO
     *
     * @else
     *
     * @brief Constructor
     *
     * Time Statistics object for profiling.
     *
     * @endif
     */
    TimeMeasure(int buflen = 100);

    /*!
     * @if jp
     *
     * @brief ԓv̌vJn
     *
     * ԓv̌vJn
     *
     * @else
     *
     * @brief Begin time measurement for time statistics
     *
     * Begin time measurement for time statistics.
     *
     * @endif
     */
    void tick();

    /*!
     * @if jp
     *
     * @brief ԓv̌vI
     *
     * ԓv̌vI
     *
     * @else
     *
     * @brief Finish time measurement for time statistics
     *
     * End of time measurement for time statistics.
     *
     * @endif
     */
    void tack();

    /*!
     * @if jp
     *
     * @brief oߎԂ擾
     *
     * oߎԂ擾
     *
     * @return TimeValue IuWFNg
     *
     * @else
     *
     * @brief Get a interval time
     *
     * Get a interval time.
     *
     * @return TimeValue object
     *
     * @endif
     */
    coil::TimeValue& interval();

    /*!
     * @if jp
     *
     * @brief v֘Af[^̏
     *
     * v֘Af[^̏
     *
     * @else
     *
     * @brief Initialize for statistics related data
     *
     * Initialize for statistics related data.
     *
     * @endif
     */
    void reset();

    /*!
     * @if jp
     *
     * @brief ԓvobt@TCY擾
     *
     * ԓvobt@TCY擾
     *
     * @return v
     *
     * @else
     *
     * @brief Get number of time measurement buffer
     *
     * Get number of time measurement buffer.
     *
     * @return Measurement count
     *
     * @endif
     */
    unsigned long int count() const;

    /*!
     * @if jp
     *
     * @brief vf[^̑v擾
     *
     * vf[^̑v擾
     *
     * @param max_interval ől [ns]
     * @param min_interval ŏl [ns]
     * @param mean_interval ϒl [ns]
     * @param stddev W΍l
     *
     * @return true: f[^, false: f[^Ȃ
     *
     * @else
     *
     * @brief Get total statistics
     *
     * Get total statistics.
     *
     * @param max_interval Max value [ns]
     * @param min_interval Min value [ns]
     * @param mean_interval Mean value [ns]
     * @param stddev Standard deviation value
     *
     * @return true: Data found, false: Data not found
     *
     * @endif
     */
    bool getStatistics(double &max_interval,
                       double &min_interval,
                       double &mean_interval,
                       double &stddev);

    /*!
     * @if jp
     *
     * @brief vʂ擾
     *
     * vʂ擾
     *
     * @return v
     *
     * @else
     *
     * @brief Get statistics result
     *
     * Get statistics result.
     *
     * @return Statistics result
     *
     * @endif
     */
    Statistics getStatistics();

  private:
    std::vector<coil::TimeValue> m_record;
    coil::TimeValue m_begin;
    coil::TimeValue m_interval;

    unsigned long int m_count;
    const unsigned long int m_countMax;
    unsigned long long int m_cpuClock;

    bool m_recurred;
  };
}; // namespace coil
#endif // COIL_TIMEMEASURE_H
